"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var image_source_1 = require("@nativescript/core/image-source/image-source");
var http_common_1 = require("./http.common");
var fs = require("tns-core-modules/file-system");
var platform_1 = require("tns-core-modules/platform");
var debugger_1 = require("tns-core-modules/debugger");
var http_common_2 = require("./http.common");
exports.HTTPFormData = http_common_2.HTTPFormData;
exports.HTTPFormDataEntry = http_common_2.HTTPFormDataEntry;
exports.ImageParseMethod = http_common_2.ImageParseMethod;
var HttpResponseEncoding;
(function (HttpResponseEncoding) {
    HttpResponseEncoding[HttpResponseEncoding["UTF8"] = 0] = "UTF8";
    HttpResponseEncoding[HttpResponseEncoding["GBK"] = 1] = "GBK";
})(HttpResponseEncoding = exports.HttpResponseEncoding || (exports.HttpResponseEncoding = {}));
function parseJSON(source) {
    var src = source.trim();
    if (src.lastIndexOf(")") === src.length - 1) {
        return JSON.parse(src.substring(src.indexOf("(") + 1, src.lastIndexOf(")")));
    }
    return JSON.parse(src);
}
var requestIdCounter = 0;
var pendingRequests = {};
var customUserAgent;
var completeCallback;
function ensureCompleteCallback() {
    if (completeCallback) {
        return;
    }
    completeCallback = new com.klippa.NativeScriptHTTP.Async.CompleteCallback({
        onComplete: function (result, context) {
            onRequestComplete(context, result);
        },
        onError: function (error, context) {
            onRequestError(error, context);
        },
    });
}
function onRequestComplete(requestId, result) {
    var callbacks = pendingRequests[requestId];
    delete pendingRequests[requestId];
    if (result.error) {
        callbacks.rejectCallback(new Error(result.error.toString()));
        return;
    }
    var headers = {};
    if (result.headers) {
        var jHeaders = result.headers;
        var length_1 = jHeaders.size();
        var pair = void 0;
        for (var i = 0; i < length_1; i++) {
            pair = jHeaders.get(i);
            addHeader(headers, pair.key, pair.value);
        }
    }
    if (global.__inspector && global.__inspector.isConnected) {
        debugger_1.NetworkAgent.responseReceived("okhttp3_" + requestId, result, headers);
    }
    callbacks.resolveCallback({
        content: {
            raw: result.raw,
            toArrayBuffer: function () { return Uint8Array.from(result.raw.toByteArray()).buffer; },
            toString: function (encoding) {
                var str;
                if (encoding) {
                    str = decodeResponse(result.raw, encoding);
                }
                else {
                    str = result.responseAsString;
                }
                if (typeof str === "string") {
                    return str;
                }
                else {
                    throw new Error("Response content may not be converted to string");
                }
            },
            toJSON: function (encoding) {
                var str;
                if (encoding) {
                    str = decodeResponse(result.raw, encoding);
                }
                else {
                    str = result.responseAsString;
                }
                return parseJSON(str);
            },
            toImage: function () {
                return new Promise(function (resolveImage, rejectImage) {
                    if (result.responseAsImage != null) {
                        resolveImage(new image_source_1.ImageSource(result.responseAsImage));
                    }
                    else {
                        rejectImage(new Error("Response content may not be converted to an Image"));
                    }
                });
            },
            toFile: function (destinationFilePath) {
                if (!destinationFilePath) {
                    destinationFilePath = http_common_1.getFilenameFromUrl(callbacks.url);
                }
                var stream;
                try {
                    var file = fs.File.fromPath(destinationFilePath);
                    var javaFile = new java.io.File(destinationFilePath);
                    stream = new java.io.FileOutputStream(javaFile);
                    stream.write(result.raw.toByteArray());
                    return file;
                }
                catch (exception) {
                    throw new Error("Cannot save file with path: " + destinationFilePath + ".");
                }
                finally {
                    if (stream) {
                        stream.close();
                    }
                }
            }
        },
        statusCode: result.statusCode,
        headers: headers
    });
}
function onRequestError(error, requestId) {
    var callbacks = pendingRequests[requestId];
    delete pendingRequests[requestId];
    if (callbacks) {
        callbacks.rejectCallback(new Error(error));
    }
}
function buildJavaOptions(options) {
    if (typeof options.url !== "string") {
        throw new Error("Http request must provide a valid url.");
    }
    var javaOptions = new com.klippa.NativeScriptHTTP.Async.Http.RequestOptions();
    javaOptions.url = options.url;
    var contentType;
    var mediaType = null;
    var userAgent;
    if (options.headers) {
        for (var key in options.headers) {
            if (key.toLowerCase() === "content-type") {
                contentType = options.headers[key];
            }
            if (key.toLowerCase() === "user-agent") {
                userAgent = options.headers[key];
            }
        }
        if (contentType) {
            mediaType = okhttp3.MediaType.parse(contentType);
        }
    }
    if (!userAgent && customUserAgent) {
        if (!options.headers) {
            options.headers = {};
        }
        options.headers["User-Agent"] = customUserAgent;
    }
    if (typeof options.method === "string") {
        javaOptions.method = options.method;
    }
    var upperCaseMethod = options.method && typeof options.method === "string" ? options.method.toUpperCase() : "GET";
    if (upperCaseMethod === "POST" || upperCaseMethod === "PATCH" || upperCaseMethod === "PUT" || upperCaseMethod === "PROPPATCH" || upperCaseMethod === "REPORT") {
        if (typeof options.content === "string" || options.content instanceof FormData) {
            if (!mediaType) {
                mediaType = okhttp3.MediaType.parse("application/x-www-form-urlencoded");
            }
            javaOptions.content = okhttp3.RequestBody.create(options.content.toString(), mediaType);
        }
        else if (options.content instanceof ArrayBuffer) {
            if (!mediaType) {
                mediaType = okhttp3.MediaType.parse("application/x-www-form-urlencoded");
            }
            var typedArray = new Uint8Array(options.content);
            var nativeBuffer = java.nio.ByteBuffer.wrap(Array.from(typedArray));
            javaOptions.content = okhttp3.RequestBody.create(nativeBuffer.array(), mediaType);
        }
        else {
            var matched = false;
            if (!matched && options.content instanceof Blob) {
                if (!mediaType && options.content.type) {
                    mediaType = okhttp3.MediaType.parse(options.content.type);
                }
                var typedArray = new Uint8Array(Blob.InternalAccessor.getBuffer(options.content).buffer.slice(0));
                var nativeBuffer = java.nio.ByteBuffer.wrap(Array.from(typedArray));
                javaOptions.content = okhttp3.RequestBody.create(nativeBuffer.array(), mediaType);
                matched = true;
            }
            if (!matched && options.content instanceof http_common_1.HTTPFormData) {
                matched = true;
                if (!mediaType) {
                    mediaType = okhttp3.MediaType.parse("multipart/form-data");
                }
                var builder_1 = new okhttp3.MultipartBody.Builder();
                builder_1.setType(mediaType);
                var contentValues = options.content;
                contentValues.forEach((function (value, key) {
                    if (typeof value === "string") {
                        builder_1.addFormDataPart(key, value);
                    }
                    else if (value instanceof ArrayBuffer) {
                        var typedArray = new Uint8Array(value);
                        var nativeBuffer = java.nio.ByteBuffer.wrap(Array.from(typedArray));
                        builder_1.addFormDataPart(key, null, okhttp3.RequestBody.create(nativeBuffer.array(), null));
                    }
                    else if (value instanceof Blob) {
                        var formDataPartMediaType = null;
                        if (value.type) {
                            formDataPartMediaType = okhttp3.MediaType.parse(value.type);
                        }
                        var filename = null;
                        if (value instanceof File) {
                            filename = value.name;
                        }
                        var typedArray = new Uint8Array(Blob.InternalAccessor.getBuffer(value).buffer.slice(0));
                        var nativeBuffer = java.nio.ByteBuffer.wrap(Array.from(typedArray));
                        builder_1.addFormDataPart(key, filename, okhttp3.RequestBody.create(nativeBuffer.array(), formDataPartMediaType));
                    }
                    else if (value instanceof http_common_1.HTTPFormDataEntry) {
                        var formDataPartMediaType = null;
                        if (value.type) {
                            formDataPartMediaType = okhttp3.MediaType.parse(value.type);
                        }
                        if (value.data instanceof ArrayBuffer) {
                            var typedArray = new Uint8Array(value.data);
                            var nativeBuffer = java.nio.ByteBuffer.wrap(Array.from(typedArray));
                            builder_1.addFormDataPart(key, value.name, okhttp3.RequestBody.create(nativeBuffer.array(), formDataPartMediaType));
                        }
                        else if (value.data instanceof Blob) {
                            var typedArray = new Uint8Array(Blob.InternalAccessor.getBuffer(value.data).buffer.slice(0));
                            var nativeBuffer = java.nio.ByteBuffer.wrap(Array.from(typedArray));
                            builder_1.addFormDataPart(key, value.name, okhttp3.RequestBody.create(nativeBuffer.array(), formDataPartMediaType));
                        }
                        else {
                            builder_1.addFormDataPart(key, value.name, okhttp3.RequestBody.create(value.data, formDataPartMediaType));
                        }
                    }
                    else {
                        builder_1.addFormDataPart(key, null, okhttp3.RequestBody.create(value, null));
                    }
                }));
                javaOptions.content = builder_1.build();
            }
            if (!matched && options.content) {
                javaOptions.content = okhttp3.RequestBody.create(options.content, mediaType);
            }
            else if (!matched) {
                javaOptions.content = okhttp3.RequestBody.create("", mediaType);
            }
        }
    }
    if (typeof options.timeout === "number") {
        javaOptions.timeout = options.timeout;
    }
    if (typeof options.dontFollowRedirects === "boolean") {
        javaOptions.dontFollowRedirects = options.dontFollowRedirects;
    }
    if (options.headers) {
        var arrayList = new java.util.ArrayList();
        var pair = com.klippa.NativeScriptHTTP.Async.Http.KeyValuePair;
        for (var key in options.headers) {
            arrayList.add(new pair(key, options.headers[key] + ""));
        }
        javaOptions.headers = arrayList;
    }
    var ourScreen = platform_1.screen.mainScreen;
    javaOptions.screenWidth = ourScreen.widthPixels;
    javaOptions.screenHeight = ourScreen.heightPixels;
    return javaOptions;
}
function request(options) {
    if (options === undefined || options === null) {
        return Promise.reject("No options given");
    }
    return new Promise(function (resolve, reject) {
        try {
            var javaOptions = buildJavaOptions(options);
            if (global.__inspector && global.__inspector.isConnected) {
                debugger_1.NetworkAgent.requestWillBeSent("okhttp3_" + requestIdCounter, options);
            }
            var callbacks = {
                url: options.url,
                resolveCallback: resolve,
                rejectCallback: reject
            };
            pendingRequests[requestIdCounter] = callbacks;
            ensureCompleteCallback();
            com.klippa.NativeScriptHTTP.Async.Http.MakeRequest(javaOptions, completeCallback, new java.lang.Integer(requestIdCounter));
            requestIdCounter++;
        }
        catch (ex) {
            reject(ex);
        }
    });
}
exports.request = request;
function decodeResponse(raw, encoding) {
    var charsetName = "UTF-8";
    if (encoding === HttpResponseEncoding.GBK) {
        charsetName = "GBK";
    }
    return raw.toString(charsetName);
}
function getString(arg) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                var str = r.content.toString();
                resolve(str);
            }
            catch (e) {
                reject(e);
            }
        }, function (e) { return reject(e); });
    });
}
exports.getString = getString;
function getJSON(arg) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                var json = r.content.toJSON();
                resolve(json);
            }
            catch (e) {
                reject(e);
            }
        }, function (e) { return reject(e); });
    });
}
exports.getJSON = getJSON;
function getImage(arg) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                resolve(r.content.toImage());
            }
            catch (err) {
                reject(err);
            }
        }, function (err) {
            reject(err);
        });
    });
}
exports.getImage = getImage;
function getFile(arg, destinationFilePath) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                var file = r.content.toFile(destinationFilePath);
                resolve(file);
            }
            catch (e) {
                reject(e);
            }
        }, function (e) { return reject(e); });
    });
}
exports.getFile = getFile;
function getBinary(arg) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                var arrayBuffer = r.content.toArrayBuffer();
                resolve(arrayBuffer);
            }
            catch (e) {
                reject(e);
            }
        }, function (e) { return reject(e); });
    });
}
exports.getBinary = getBinary;
function addHeader(headers, key, value) {
    if (!headers[key]) {
        headers[key] = value;
    }
    else if (Array.isArray(headers[key])) {
        headers[key].push(value);
    }
    else {
        var values = [headers[key]];
        values.push(value);
        headers[key] = values;
    }
}
exports.addHeader = addHeader;
function setImageParseMethod(imageParseMethod) {
    if (imageParseMethod === http_common_1.ImageParseMethod.ALWAYS) {
        com.klippa.NativeScriptHTTP.Async.Http.SetImageParseMethod(com.klippa.NativeScriptHTTP.Async.Http.ImageParseMethod.ALWAYS);
    }
    else if (imageParseMethod === http_common_1.ImageParseMethod.CONTENTTYPE) {
        com.klippa.NativeScriptHTTP.Async.Http.SetImageParseMethod(com.klippa.NativeScriptHTTP.Async.Http.ImageParseMethod.CONTENTTYPE);
    }
    else if (imageParseMethod === http_common_1.ImageParseMethod.NEVER) {
        com.klippa.NativeScriptHTTP.Async.Http.SetImageParseMethod(com.klippa.NativeScriptHTTP.Async.Http.ImageParseMethod.NEVER);
    }
}
exports.setImageParseMethod = setImageParseMethod;
function setConcurrencyLimits(maxRequests, maxRequestsPerHost) {
    com.klippa.NativeScriptHTTP.Async.Http.SetConcurrencyLimits(maxRequests, maxRequestsPerHost);
}
exports.setConcurrencyLimits = setConcurrencyLimits;
function clearCookies() {
    com.klippa.NativeScriptHTTP.Async.Http.ClearCookies();
}
exports.clearCookies = clearCookies;
function setUserAgent(userAgent) {
    customUserAgent = userAgent;
}
exports.setUserAgent = setUserAgent;
function certificatePinningAdd(pattern, hashes) {
    var newHashes = hashes.map(function (value) { return "sha256/" + value; });
    com.klippa.NativeScriptHTTP.Async.Http.PinCertificate(pattern, newHashes);
}
exports.certificatePinningAdd = certificatePinningAdd;
function certificatePinningClear() {
    com.klippa.NativeScriptHTTP.Async.Http.RemoveCertificatePins();
}
exports.certificatePinningClear = certificatePinningClear;
//# sourceMappingURL=http.android.js.map